"use strict";
/*
 * Copyright 2021-2022 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TcpServerTransport = void 0;
var rsocket_core_1 = require("rsocket-core");
var net_1 = __importDefault(require("net"));
var TcpDuplexConnection_1 = require("./TcpDuplexConnection");
var TcpServerTransport = /** @class */ (function () {
    function TcpServerTransport(options) {
        var _a;
        this.serverOptions = options.serverOptions;
        this.listenOptions = options.listenOptions;
        this.serverCreator =
            (_a = options.socketCreator) !== null && _a !== void 0 ? _a : (function (options) { return new net_1.default.Server(options); });
    }
    TcpServerTransport.prototype.bind = function (connectionAcceptor, multiplexerDemultiplexerFactory) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var socketServer = _this.serverCreator(_this.serverOptions);
            var earlyCloseListener = function (error) {
                reject(error);
            };
            socketServer.addListener("close", earlyCloseListener);
            socketServer.addListener("error", earlyCloseListener);
            socketServer.addListener("listening", function () {
                var serverCloseable = new ServerCloseable(socketServer);
                var connectionListener = function (socket) {
                    TcpDuplexConnection_1.TcpDuplexConnection.create(socket, connectionAcceptor, multiplexerDemultiplexerFactory);
                };
                var closeListener = function (error) {
                    serverCloseable.close(error);
                };
                socketServer.addListener("connection", connectionListener);
                socketServer.removeListener("close", earlyCloseListener);
                socketServer.removeListener("error", earlyCloseListener);
                socketServer.addListener("close", closeListener);
                socketServer.addListener("error", closeListener);
                resolve(serverCloseable);
            });
            socketServer.listen(_this.listenOptions);
        });
    };
    return TcpServerTransport;
}());
exports.TcpServerTransport = TcpServerTransport;
var ServerCloseable = /** @class */ (function (_super) {
    __extends(ServerCloseable, _super);
    function ServerCloseable(server) {
        var _this = _super.call(this) || this;
        _this.server = server;
        return _this;
    }
    ServerCloseable.prototype.close = function (error) {
        if (this.done) {
            _super.prototype.close.call(this, error);
            return;
        }
        this.server.close();
        _super.prototype.close.call(this);
    };
    return ServerCloseable;
}(rsocket_core_1.Deferred));
//# sourceMappingURL=TcpServerTransport.js.map